@@ -134,7 +134,16 @@ extern int yydebug;
     VAR_MULTI_MASTER_CHECK = 344,
     VAR_MINIMAL_RESPONSES = 345,
     VAR_REFUSE_ANY = 346,
-    VAR_USE_SYSTEMD = 347
+    VAR_USE_SYSTEMD = 347,
+    VAR_DNSTAP = 348,
+    VAR_DNSTAP_ENABLE = 349,
+    VAR_DNSTAP_SOCKET_PATH = 350,
+    VAR_DNSTAP_SEND_IDENTITY = 351,
+    VAR_DNSTAP_SEND_VERSION = 352,
+    VAR_DNSTAP_IDENTITY = 353,
+    VAR_DNSTAP_VERSION = 354,
+    VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES = 355,
+    VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES = 356
   };
 #endif
 /* Tokens.  */
@@ -228,6 +237,15 @@ extern int yydebug;
 #define VAR_MINIMAL_RESPONSES 345
 #define VAR_REFUSE_ANY 346
 #define VAR_USE_SYSTEMD 347
+#define VAR_DNSTAP 348
+#define VAR_DNSTAP_ENABLE 349
+#define VAR_DNSTAP_SOCKET_PATH 350
+#define VAR_DNSTAP_SEND_IDENTITY 351
+#define VAR_DNSTAP_SEND_VERSION 352
+#define VAR_DNSTAP_IDENTITY 353
+#define VAR_DNSTAP_VERSION 354
+#define VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES 355
+#define VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES 356
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -238,7 +256,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 242 "configparser.h" /* yacc.c:1909  */
+#line 260 "configparser.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
