@@ -462,7 +462,7 @@ nsec3_delete_rr_trigger(namedb_type* db, rr_type* rr, zone_type* zone,
 		/* clear trees, wipe hashes, wipe precompile */
 		nsec3_clear_precompile(db, zone);
 		/* pick up new nsec3param (from udb, or avoid deleted rr) */
-		nsec3_find_zone_param(db, zone, udbz, rr);
+		nsec3_find_zone_param(db, zone, udbz, rr, 0);
 		/* if no more NSEC3, done */
 		if(!zone->nsec3_param)
 			return;
@@ -568,7 +568,7 @@ nsec3_add_rr_trigger(namedb_type* db, rr_type* rr, zone_type* zone,
 		prehash_add(db->domains, rr->owner);
 	} else if(!zone->nsec3_param && rr->type == TYPE_NSEC3PARAM) {
 		/* see if this means NSEC3 chain can be used */
-		nsec3_find_zone_param(db, zone, udbz, NULL);
+		nsec3_find_zone_param(db, zone, udbz, NULL, 0);
 		if(!zone->nsec3_param)
 			return;
 		nsec3_zone_trees_create(db->region, zone);
@@ -1138,15 +1138,12 @@ apply_ixfr(namedb_type* db, FILE *in, const char* zone, uint32_t serialno,
 		if(*rr_count == 1 && type != TYPE_SOA) {
 			/* second RR: if not SOA: this is an AXFR; delete all zone contents */
 #ifdef NSEC3
-			nsec3_hash_tree_clear(zone_db);
+			nsec3_clear_precompile(db, zone_db);
+			zone_db->nsec3_param = NULL;
 #endif
 			delete_zone_rrs(db, zone_db);
 			if(db->udb)
 				udb_zone_clear(db->udb, udbz);
-#ifdef NSEC3
-			nsec3_clear_precompile(db, zone_db);
-			zone_db->nsec3_param = NULL;
-#endif /* NSEC3 */
 			/* add everything else (incl end SOA) */
 			*delete_mode = 0;
 			*is_axfr = 1;
@@ -1169,15 +1166,12 @@ apply_ixfr(namedb_type* db, FILE *in, const char* zone, uint32_t serialno,
 			if(thisserial == serialno) {
 				/* AXFR */
 #ifdef NSEC3
-				nsec3_hash_tree_clear(zone_db);
+				nsec3_clear_precompile(db, zone_db);
+				zone_db->nsec3_param = NULL;
 #endif
 				delete_zone_rrs(db, zone_db);
 				if(db->udb)
 					udb_zone_clear(db->udb, udbz);
-#ifdef NSEC3
-				nsec3_clear_precompile(db, zone_db);
-				zone_db->nsec3_param = NULL;
-#endif /* NSEC3 */
 				*delete_mode = 0;
 				*is_axfr = 1;
 			}
@@ -1913,7 +1907,8 @@ task_process_del_zone(struct nsd* nsd, struct task_list_d* task)
 		return;
 
 #ifdef NSEC3
-	nsec3_hash_tree_clear(zone);
+	nsec3_clear_precompile(nsd->db, zone);
+	zone->nsec3_param = NULL;
 #endif
 	delete_zone_rrs(nsd->db, zone);
 	if(nsd->db->udb) {
@@ -1924,10 +1919,6 @@ task_process_del_zone(struct nsd* nsd, struct task_list_d* task)
 			udb_ptr_unlink(&udbz, nsd->db->udb);
 		}
 	}
-#ifdef NSEC3
-	nsec3_clear_precompile(nsd->db, zone);
-	zone->nsec3_param = NULL;
-#endif /* NSEC3 */
 
 	/* remove from zonetree, apex, soa */
 	zopt = zone->opts;
