@@ -1,19 +1,21 @@
 /*
  * edns.c -- EDNS definitions (RFC 2671).
  *
- * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
  */
 
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 
 #include "dns.h"
 #include "edns.h"
+#include "nsd.h"
+#include "query.h"
 
 void
 edns_init_data(edns_data_type *data, uint16_t max_length)
@@ -53,12 +55,42 @@ edns_init_record(edns_record_type *edns)
 	edns->status = EDNS_NOT_PRESENT;
 	edns->position = 0;
 	edns->maxlen = 0;
+	edns->opt_reserved_space = 0;
 	edns->dnssec_ok = 0;
 	edns->nsid = 0;
 }
 
+/** handle a single edns option in the query */
+static int
+edns_handle_option(uint16_t optcode, uint16_t optlen, buffer_type* packet,
+	edns_record_type* edns, struct query* query, nsd_type* nsd)
+{
+	(void) query; /* in case edns options need the query structure */
+	/* handle opt code and read the optlen bytes from the packet */
+	switch(optcode) {
+	case NSID_CODE:
+		/* is NSID enabled? */
+		if(nsd->nsid_len > 0) {
+			edns->nsid = 1;
+			/* we have to check optlen, and move the buffer along */
+			buffer_skip(packet, optlen);
+			/* in the reply we need space for optcode+optlen+nsid_bytes */
+			edns->opt_reserved_space += OPT_HDR + nsd->nsid_len;
+		} else {
+			/* ignore option */
+			buffer_skip(packet, optlen);
+		}
+		break;
+	default:
+		buffer_skip(packet, optlen);
+		break;
+	}
+	return 1;
+}
+
 int
-edns_parse_record(edns_record_type *edns, buffer_type *packet)
+edns_parse_record(edns_record_type *edns, buffer_type *packet,
+	query_type* query, nsd_type* nsd)
 {
 	/* OPT record type... */
 	uint8_t  opt_owner;
@@ -67,7 +99,6 @@ edns_parse_record(edns_record_type *edns, buffer_type *packet)
 	uint8_t  opt_version;
 	uint16_t opt_flags;
 	uint16_t opt_rdlen;
-	uint16_t opt_nsid;
 
 	edns->position = buffer_position(packet);
 
@@ -89,16 +120,27 @@ edns_parse_record(edns_record_type *edns, buffer_type *packet)
 	opt_rdlen = buffer_read_u16(packet);
 
 	if (opt_version != 0) {
+		/* The only error is VERSION not implemented */
 		edns->status = EDNS_ERROR;
 		return 1;
 	}
 
 	if (opt_rdlen > 0) {
-		/* there is more to come, read opt code
-		 * should be NSID - there are no others */
-		opt_nsid = buffer_read_u16(packet);
-		edns->nsid = (opt_nsid == NSID_CODE);
-		/* extra check for the value */
+		if(!buffer_available(packet, opt_rdlen))
+			return 0;
+		/* there is more to come, read opt code */
+		while(opt_rdlen >= 4) {
+			uint16_t optcode = buffer_read_u16(packet);
+			uint16_t optlen = buffer_read_u16(packet);
+			if(opt_rdlen < 4+optlen)
+				return 0; /* opt too long, formerr */
+			opt_rdlen -= (4+optlen);
+			if(!edns_handle_option(optcode, optlen, packet,
+				edns, query, nsd))
+				return 0;
+		}
+		if(opt_rdlen != 0)
+			return 0;
 	}
 
 	edns->status = EDNS_OK;
@@ -111,5 +153,5 @@ size_t
 edns_reserved_space(edns_record_type *edns)
 {
 	/* MIEK; when a pkt is too large?? */
-	return edns->status == EDNS_NOT_PRESENT ? 0 : (OPT_LEN + OPT_RDATA);
+	return edns->status == EDNS_NOT_PRESENT ? 0 : (OPT_LEN + OPT_RDATA + edns->opt_reserved_space);
 }
