@@ -25,7 +25,6 @@
 #include "udbzone.h"
 #include "util.h"
 
-static void error(const char *format, ...) ATTR_FORMAT(printf, 1, 2);
 struct nsd nsd;
 
 /*
@@ -40,20 +39,6 @@ usage (void)
 		PACKAGE_VERSION, PACKAGE_BUGREPORT);
 }
 
-/*
- * Something went wrong, give error messages and exit.
- *
- */
-static void
-error(const char *format, ...)
-{
-	va_list args;
-	va_start(args, format);
-	log_vmsg(LOG_ERR, format, args);
-	va_end(args);
-	exit(1);
-}
-
 /* zone memory structure */
 struct zone_mem {
 	/* size of data (allocated in db.region) */
