@@ -13,6 +13,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <sys/uio.h>
 #include "nsd.h"
 #include "xfrd-tcp.h"
 #include "buffer.h"
@@ -898,6 +899,7 @@ xfrd_tcp_read(struct xfrd_tcp_pipeline* tp)
 			tp->id[zone->query_id] = TCP_NULL_SKIP;
 			tp->num_skip++;
 			/* fall through to remove zone from tp */
+			/* fallthrough */
 		case xfrd_packet_transfer:
 			if(zone->zone_options->pattern->multi_master_check) {
 				xfrd_tcp_release(xfrd->tcp_set, zone);
