@@ -805,6 +805,7 @@ precsize_aton (char *cp, char **endptr)
 	}
 
 	if(mval >= poweroften[7]) {
+		assert(poweroften[7] != 0);
 		/* integer overflow possible for *100 */
 		mantissa = mval / poweroften[7];
 		exponent = 9; /* max */
@@ -816,6 +817,7 @@ precsize_aton (char *cp, char **endptr)
 			if (cmval < poweroften[exponent+1])
 				break;
 
+		assert(poweroften[exponent] != 0);
 		mantissa = cmval / poweroften[exponent];
 	}
 	if (mantissa > 9)
@@ -1260,6 +1262,8 @@ parse_unknown_rdata(uint16_t type, uint16_t *wireformat)
 			zadd_rdata_wireformat(rdatas[i].data);
 		}
 	}
+	region_recycle(parser->region, rdatas,
+		rdata_count*sizeof(rdata_atom_type));
 }
 
 
@@ -1626,7 +1630,9 @@ zonec_read(const char* name, const char* zonefile, zone_type* zone)
 			name, domain_to_string(
 			parser->current_zone->soa_rrset->rrs[0].owner));
 	}
+	region_free_all(parser->rr_region);
 
+	parser_flush();
 	fclose(yyin);
 	if(!zone_is_slave(zone->opts))
 		check_dname(zone);
@@ -1666,6 +1672,9 @@ zonec_desetup_parser(void)
 		 * region_recycle(parser->region, (void*)error_domain, 1); */
 		/* clear memory for exit, but this is not portable to
 		 * other versions of lex. yylex_destroy(); */
+#ifdef MEMCLEAN /* OS collects memory pages */
+		yylex_destroy();
+#endif
 	}
 }
 
@@ -1718,6 +1727,8 @@ zonec_parse_string(region_type* region, domain_table_type* domains,
 	/* remove origin if it was not used during the parse */
 	if(parser->origin != error_domain)
 		domain_table_deldomain(parser->db, parser->origin);
+	region_free_all(parser->rr_region);
 	zonec_desetup_string_parser();
+	parser_flush();
 	return errors;
 }
